% IKINE	Calcula la cinemtica inversa del manipulador
%
%	Q = IKINE(DH, STOL, ILIMIT, T)
%	Q = IKINE(DH, STOL, ILIMIT, T, Q0)
%	Q = IKINE(DH, STOL, ILIMIT, T, Q0, M)
%
%	Devuelve los valores de las variables articulares necesarios para que el efector
%	final del manipulador tenga la posicin y orientacin dadas por la transformacin
%	T. La solucin del problema cinemtico inverso no es nica en general, y es posi-
%	ble que para una misma orientacin y posicin deseadas, se obtengan soluciones dis-
%	tintas en funcin del vector inicial de variables articulares (Q0) que se le pase
%	a IKINE.
%
%	Es posible usar la funcin para que devuelva las variables articulares correspon-
%	dientes a una sla posicin y orientacin, o bien a una trayectoria de posiciones
%	y orientaciones. Eso depender del formato del parmetro T:
%
%	-	Si T es una transformacin homognea, entonces IKINE devuelve un vector
%		fila(Q) con las variables articulares correspondientes a la posicin y
%		orientacin indicadas en la matriz T.
%
%	-	Si T es una trayectoria de transformaciones homogneas, entonces el re-
%		sultado ser una matriz (Q), en la que la fila i-sima contendr las va-
%		riables articulares correspondientes a la transformacin T (:, :, i ).
%		La estimacin inicial para Q en cada paso se toma de la solucin obtenida
%		en el paso anterior.
%
%	Sea cual sea el formato de T, la estimacin inicial para el vector de variables
%	articulares ser la dada en el parmetro Q0 (puede ser una columna o una fila), y
%	en el caso de que no se lo demos, asume que es el vector nulo. 
%
%	Para el caso de un manipulador con menos de 6 grados de libertad el efector final
%	no podr alcanzar algunas posiciones y orientaciones. Esto normalmente lleva a una
%	no convergencia de IKINE. Una solucin consiste en especificar un vector (fila o
%	columna) de pesos (M), cuyos elementos sern 0 para aquellos grados de libertad que
%	en cartesianas estn restringidos, y 1 en otro caso. Los elementos de M se corres-
%	ponden con las traslaciones a lo largo de los ejes X, Y y Z, y con las rotaciones en
%	torno a los ejes X, Y y Z. 
%	Por ejemplo si el manipulador no se puede desplazar a lo largo del eje Z, ni rotar en 
%	torno a los ejes X e Y, M deber ser el vector [1 1 0 0 0 1].
%	El nmero de elementos no nulos debe ser igual al nmero de grados de libertad del ro-
%	bot.
%
%	ILIMIT es el nmero mximo de iteraciones que se ejecutarn en busca de una solucin 
%	(un valor usual es 1000).
% 
%	STOL ser la mxima diferencia que se admitir entre la transformacin correspondiente
%	a las variables articulares solucin y la transformacin con la posicin y orientacin
%	especificadas (un valor usual es 1e-6). Dicha diferencia se mide haciendo uso de la 
%	funcin TR2DIFF.
%
%	Ver tambin FKINE, JACOB0, TR2DIFF.
	
%  Copyright (C)	1999 Peter Corke
%						2001 Ivn Maza


function qt = ikine(dh, stol, ilimit, tr, q, m)

	n = numrows(dh);

	if nargin == 4,
		q = zeros(n, 1);
	else
		q = q(:);
	end
	if nargin == 6,
		m = m(:);
      if numrows(m) ~= 6 
         error('El vector mscara debe tener 6 elementos')
		end
	else
      if numrows(dh) < 6,
         disp('Para un manipulador con menos de 6 DOF se debe pasar como parmetro un vector mscara');
		end
		m = ones(6, 1);
	end
		

	tcount = 0;
	if ishomog(tr),		% Caso de que se especifique una sla transformacin
		nm = 1;
		count = 0;
		while nm > stol,
			e = tr2diff(fkine(dh, q'), tr) .* m;
			dq = pinv( jacob0(dh, q') )*e;
         q = q + dq;                 
			nm = norm(dq);
			count = count+1;
         if count > ilimit,
            error('Parece que no hay convergencia')
			end
		end
		qt = q';
	else			% Caso de que se trate de una trayectoria de transformaciones
      np = size(tr,3);
      qt=[];
		for i=1:np,
			nm = 1;
         T = tr(:,:,i);
			count = 0;
			while nm > stol,
				e = tr2diff(fkine(dh, q'), T) .* m;
            dq = pinv( jacob0(dh, q') ) * e;
            q = q + dq;
				nm = norm(dq);
				count = count+1;
            if count > ilimit,
         		error('Parece que no hay convergencia')
				end
			end
			qt = [qt; q'];
			tcount = tcount + count;
		end
	end
